#!/usr/bin/perl

# up2.0.6-2.0.7.pl
# upgrades Tallyman from version 2.0.6 to 2.0.7
# (which, incidentally, only entails setting the "Tallyman Version"
#  KNAR entry to "2.0.7")

# 13 Jun 2000
# Jason E. Holt
# for Akopia, Inc.
# jason@akopia.com


$| = 1;

use Getopt::Long;

$usage = <<EOF;
usage:

$0  [--help]  [--terse]  [--force] 
                    [--tallyman-sid=STORE ID]  [--tallyman-path=PATH]
                    [--not-really]  [--upgrade-all]
EOF

# Setting TALLYMAN_PATH:
# --tallyman-path has first precedence, then 
# the environment var, then ../..

################### Parse the options
if(!GetOptions('terse'         => \$TERSE,
		  'force'           => \$FORCE,
		  'tallyman-sid=s'  => \$TM_SID,
		  'tallyman-path=s' => \$ENV{TALLYMAN_PATH},
		  'not-really'      => \$NOT_REALLY,
		  'upgrade-all'     => \$DO_ALL,
		  'help'            => \$HELP
		  ) || $HELP) {
print <<EOF;

$usage

Upgrades a Tallyman 2.0.6 installation to 2.0.7.

Option descriptions:

    --help           Display this message
    --terse          Skip details while upgrading
    --force          Upgrade even if the version appears to be wrong
    --tallyman-sid   Specify a specific store id to upgrade
    --tallyman-path  Specify the base directory of a Tallyman installation
    --not-really     Just pretend to upgrade.
    --upgrade-all    Non-interactively attempt to upgrade all installed stores

EOF
    exit 1;
}
    
$VERBOSE = ($TERSE ? 0 : 1);

################### Load the tm libs
unless($ENV{TALLYMAN_PATH}) {
    chdir "../..";
    $ENV{TALLYMAN_PATH} = `pwd`;
    chomp $ENV{TALLYMAN_PATH};
    print "\nUsing '$ENV{TALLYMAN_PATH}' for TALLYMAN_PATH env var.\n\n";
}

unless(eval "
             use lib '$ENV{TALLYMAN_PATH}/lib';
             use KNAR;
             use ACCESS;
             1;
            ") {

    print <<EOF;

The following error occurred when trying to load the Tallyman
libraries.  You are in tools/upgrade/, aren't you? 

$@

EOF

    exit 1;
}

$TM_SID eq 'default' && die "'default' isn't a real store.";
$TM_SID eq 'CVS'     && die "'CVS' isn't a real store.";

chdir("$ENV{TALLYMAN_PATH}/stores") || die 
    "Can't cd to $ENV{TALLYMAN_PATH}/stores: $!";

@sids = ($TM_SID || <*>);
foreach $sid (@sids) {

    next if $sid eq "default";
    next if $sid eq "CVS";

    $ENV{TALLYMAN_SID} = $sid;

    unless($DO_ALL || $TM_SID) {
	print "Attempt to upgrade store '$sid' [y/n]: ";
	my $answer = <>;
	unless ($answer =~ /^[yY]/) {
	    print "Skipping.\n";
	    next;
	}
    }

    print "Store '$sid': ";

    ################### Verify tm version
    $ver = KNAR::knar_entry_get('Tallyman Version');
    if(!$FORCE && ($ver ne "2.0.6")) {

	print "skipping (wrong version).\n";
	$VERBOSE && print <<EOF;

The 'Tallyman Version' entry in the KNAR table suggests that you're
running version '$ver' of Tallyman.  This script only knows how to 
upgrade from version 2.0.6 to 2.0.7.  Specify --force to attempt an 
upgrade anyway (at your own risk).

If you're upgrading from version 2.0a3 or earlier, first run up2.0a3-2.0b5.pl 
to upgrade the tables to 2.0b5.  Then run this script again.

EOF
        next;
    } else {
	print "attempting to upgrade.\n";
    }

    &do_upgrade($VERBOSE, $NOT_REALLY);
}

print "\nExit.\n";


sub do_upgrade {
    my($VERBOSE, $NOT_REALLY) = @_;

    # Real tough upgrade, eh?

    DBLIB::db_transact_begin() unless $NOT_REALLY;

    $NOT_REALLY && $VERBOSE && print "(not really) ";
    $VERBOSE && print "Setting Tallyman Version to 2.0.7\n";
    KNAR::knar_entry_set_by_name("Tallyman Version", '2.0.7', '') 
	unless $NOT_REALLY;

    DBLIB::db_transact_end() unless $NOT_REALLY;
}

